package widebase.stream.socket.rq

import scala.collection.mutable. { ArrayBuffer, LinkedHashMap, Map }
import scala.concurrent.Lock

import vario.filter.StreamFilter
import vario.filter.StreamFilter.StreamFilter

import widebase.stream.codec.rq. { BrokerEncoder, RequestDecoder }
import widebase.stream.handler.AuthMap

import widebase.stream.handler.rq. {

  ConsumerWriter,
  PersistenceWriter,
  PublishingHandler,
  SubscribingHandler

}

import widebase.stream.socket.ServerLike

/** A broker.
 *
 * @param path to rollback database
 * @param jaas entry name
 * @param auths procedure based authorizations
 * @param interval to publish, default zero-latency
 *
 * @author myst3r10n
 */
class Broker(
  path: String,
  protected val jaas: String,
  protected val auths: AuthMap,
  interval: Int = 0)
  extends ServerLike {

  /** Lock objects. */
  protected val lock = new Lock

  /** Holds persistence wrtiers. */
  protected val persistences = Map[String, PersistenceWriter]()

  /** Holds subscription wrtiers. */
  protected val subscriptions = Map[String, ArrayBuffer[ConsumerWriter]]()

  override protected def decoder = new RequestDecoder
  override protected def encoder = new BrokerEncoder

  override protected def handlers = LinkedHashMap(
    "publishing" -> new PublishingHandler(
      path,
      interval * 1000L,
      lock,
      persistences,
      subscriptions),
    "subscribing" -> new SubscribingHandler(lock, persistences, subscriptions))

}

