package widebase.stream.socket.rq

import java.io.IOException
import java.util.concurrent.TimeUnit

import widebase.db.table.Table

import widebase.stream.codec. {

  DoneMessage,
  ForbiddenMessage,
  LoginRequiredMessage,
  UnauthorizedMessage

}

import widebase.stream.codec.rq. {

  BrokerDecoder,
  ProducerEncoder,
  PublishMessage

}

import widebase.stream.socket.ClientLike

/** A producer.
 *
 * @param host of broker
 * @param port of broker
 * @param username to login
 * @param password to login
 *
 * @author myst3r10n
 */
class Producer(val host: String, val port: Int) extends ClientLike {

  /** Decoder. */
  protected def decoder = new BrokerDecoder

  /** Encoders. */
  protected def encoder = new ProducerEncoder

  /** Handlers. */
  protected def handlers = null

  override def login(
    username: String,
    password: String,
    timeout: Int = 60): Producer = {

    val future = connect(host, port).awaitUninterruptibly

    if(!future.isSuccess)
      throw new IOException(future.getCause)

    try {

      super.login(username, password, timeout).asInstanceOf[Producer]

    } catch {

      case e =>
        val future = disconnect.awaitUninterruptibly

        if(!future.isSuccess)
          throw new IOException(future.getCause)

        throw e

    }
  }

  /** Publish records.
   *
   * @param name of table
   * @param records to publish
   * @param timeout of response
   *
   * @return itself
   */
  def publish(name: String, records: Table, timeout: Int = 60) = {

    val future = channel.write(new PublishMessage(name, records.toBytes())).awaitUninterruptibly

    if(!future.isSuccess)
      throw new IOException(future.getCause)

    val message = reader.read(timeout, TimeUnit.SECONDS)

    if(message == null)
      throw new IOException(new NullPointerException)

    message match {

      case message: DoneMessage =>
      case message: ForbiddenMessage => throw new ForbiddenMessage
      case message: LoginRequiredMessage => throw new LoginRequiredMessage
      case message: UnauthorizedMessage => throw new UnauthorizedMessage

    }

    this

  }
}

