package widebase.stream.socket

import vario.filter. { CompressionLevel, StreamFilter }

import widebase.stream.handler.AuthMap
import widebase.stream.handler.rq.RecordListener

/** Socket RQ package
 *
 * @author myst3r10n
 */
package object rq {

  import vario.filter.StreamFilter.StreamFilter

  /** Create [[widebase.stream.socket.rq.Broker]] instance.
   *
   * @param path to rollback database
   * @param jaas entry name
   * @param auths procedure based authorizations
   * @param interval to publish
   * @param filter of table file or socket I/O
   * @param level0 of outbound socket compression
   *
   * @return instance of [[widebase.stream.socket.rq.Broker]]
   */
  def broker(
    path: String,
    jaas: String,
    auths: AuthMap,
    interval: Int = 0,
    filter: String = "",
    level0: Int = CompressionLevel.Default) =
    new Broker(path, jaas, auths, interval) {

      override def inFilter =
        if(filter.contains("rg"))
          StreamFilter.Gzip
        else if(filter.contains("rz"))
          StreamFilter.Zlib
        else
          StreamFilter.None

      override def outFilter =
        if(filter.contains("wg"))
          StreamFilter.Gzip
        else if(filter.contains("wz"))
          StreamFilter.Zlib
        else
          StreamFilter.None

      override def level = level0

    }

  /** Create [[widebase.stream.socket.rq.Consumer]] instance.
   *
   * @param listener of consumer
   * @param host of server
   * @param port of server
   * @param filter of socket I/O
   * @param level0 of outbound socket compression
   *
   * @return instance of [[widebase.stream.socket.rq.Consumer]]
   */
  def consumer(
    listener: RecordListener,
    host: String,
    port: Int,
    filter: String = "",
    level0: Int = CompressionLevel.Default) =
    new Consumer(listener, host, port) {

      override def inFilter =
        if(filter.contains("rg"))
          StreamFilter.Gzip
        else if(filter.contains("rz"))
          StreamFilter.Zlib
        else
          StreamFilter.None

      override def outFilter =
        if(filter.contains("wg"))
          StreamFilter.Gzip
        else if(filter.contains("wz"))
          StreamFilter.Zlib
        else
          StreamFilter.None

      override def level = level0

    }

  /** Create [[widebase.stream.socket.rq.Producer]] instance.
   *
   * @param host of server
   * @param port of server
   * @param filter of socket I/O
   * @param level0 of outbound socket compression
   *
   * @return instance of [[widebase.stream.socket.rq.Producer]]
   */
  def producer(
    host: String,
    port: Int,
    filter: String = "",
    level0: Int = CompressionLevel.Default) =
    new Producer(host, port) {

      override def inFilter =
        if(filter.contains("rg"))
          StreamFilter.Gzip
        else if(filter.contains("rz"))
          StreamFilter.Zlib
        else
          StreamFilter.None

      override def outFilter =
        if(filter.contains("wg"))
          StreamFilter.Gzip
        else if(filter.contains("wz"))
          StreamFilter.Zlib
        else
          StreamFilter.None

      override def level = level0

    }
}

